from sequence_data.io.gff import gff_importer,dict_from_attrstr
from sequence_data import bionumbers
import re,sys
from numpy import *
from optparse import *
usage = r"""
usage: [cat input.gff] %prog [options] > mir.energies

reads GFF with scored miRNA binding sites, computes E_bind, 
sums up the frequencies and outputs distribution of binding energies.
Can also report the binding sites per gene.
"""

parser = OptionParser(usage=usage)
parser.add_option("-z","--zamore",dest="zamore",default=False,action="store_true",help="compute semi-empirial (Wee et al. 2012 derived) energies (default is RNAduplex)")
parser.add_option("-e","--expression",dest="expression",default='hek_poly_expression.fpkm',help="table with expression data for each gene (default=HEK293 polyA mRNA-seq)")
parser.add_option("-b","--bins",dest="bins",default=35,help="number of binding energy bins (default=35)")
parser.add_option("-M","--mrna",dest="mrna",default=250E3,type=float,help="total number of mRNA molecules (default=250,000)")
parser.add_option("-p","--pdf",dest="pdf",default="",help="save distribution as PDF")
parser.add_option("-g","--genes",dest="genes",default=False,action="store_true",help="output binding sites for each gene with energy")
parser.add_option("-a","--all_energies",dest="all_energies",default=False,action="store_true",help="don't bin but report each distinct binding energy")

options,args = parser.parse_args()

def load_expression(fname):
    expression = {}
    total = 0.
    for line in file(fname):
        gene,fpkm = line.split('\t')
        fpkm = float(fpkm)
        expression[gene] = fpkm
        total += fpkm

    print "# total FPKM=%f, assuming %d mRNAs per cell, this gives 1FPKM ~ %.2f mRNAs" % (total,options.mrna,options.mrna/float(total))
    return expression,total

expressed_genes,total_fpkm = load_expression(options.expression)

energies = []
freqs = []
from collections import defaultdict
all_energies = defaultdict(float)

for i,gff in enumerate(gff_importer(sys.stdin)):
    attrs = dict_from_attrstr(gff.attr_str)

    # find all genes that contain this binding site (TODO: exonic only!)
    for M in re.finditer(r'gene/(?P<gene>\S+?)[,\"]',attrs['SENSE']):
        gene = M.groups()[0]
        if not gene in expressed_genes:
            continue
        
        freq = expressed_genes[gene] / total_fpkm
        if options.zamore:
            E = bionumbers.zamore_energy(attrs['struct'])
        else:
            E = float(gff.score)
        energies.append(E)
        all_energies[E] += freq

        if options.genes:
            sys.stderr.write('%s\t%f\n' % (gene,energies[-1]))

        freqs.append(freq)

    if not (i+1) % 1000:
        print "#scanned %dk sites" % (i/1000)

energies = array(energies)

# site concentrations in microMolars, assuming given mRNA copy numbers and standard cell volume
copies = array(freqs)*options.mrna
#conc = copies/bionumbers.c0*1e6 

if options.all_energies:
    y = []
    x_ = []
    for E in sorted(all_energies.keys()):
        x_.append(E)
        y.append(all_energies[E])
    
    y = array(y)*options.mrna
    x_ = array(x_)

else:
    y,x = pylab.histogram(energies,bins=30,weights=copies)
    x_ = (x[:-1] + x[1:])/2.

dist_e = (x_,y)

import matplotlib
matplotlib.use('pdf')
import pylab

if options.pdf:
    plot(x_,y,linestyle='step',linewidth=2,color="k")
    xlabel(r"binding energy $[\frac{kcal}{mol}]$")
    ylabel(r"abundance $[\frac{copies}{cell}]$")
    pylab.savefig(options.pdf)

print "# total binding sites per cell %d" % (y.sum())    
print "# energy\tcopies/cell\tconc_in_nM"
for e,n in zip(x_,y):
    c = n/bionumbers.c0*1e9
    print "%f\t%.1f\t%e" % (e,n,c)
