#!/usr/bin/env python
import sys
from numpy import *
from pylab import *
from optparse import *

usage = """
  %prog [options]

Models a well-mixed cellular environment with many binding sites
and a non-catalytic regulator, in equilibrium.
Here, the focus is on miR20a binding to the mRNAs expressed in monocytes, 
using expression data from Mullokandov et al. The binding energy distribution 
of miR20a-loaded RISC to different types of binding sites is derived from 
the biochemical data published in Wee et al. 2012.
"""

parser = OptionParser(usage=usage)
parser.add_option("-M","--mRNA",dest="M",default=250000.,type=float,help="number of mRNAs per cell (default=250000)")
parser.add_option("-e","--energies",dest="energies",default="monocyte_mir20a_sites.csv",help="file with binding energy distribution")
parser.add_option("-T","--temperature",dest="T",type=float,default=37,help="temperature in Celsius (default=37)")
parser.add_option("-s","--simple",dest="simple",action="store_true",default=False,help="use simple counts of 6,7,8mers instead of the energy distribution")
parser.add_option("-m","--mirnas",dest="mirnas",default="monocyte_mirna_counts.csv",help="miRNA expression profile (read counts)")
parser.add_option("-R","--RISC",dest="RISC",default=150000.,type=float,help="copy number of RISC/cell (default=150000)")
parser.add_option("-f","--format",dest="format",default="svg",choices=["svg","pdf","png"],help="file format for saving plots (default=svg) [svg,pdf,png]")

options,args = parser.parse_args()


class BioNumbers(object):
    """
    This is not really a class. We just abuse it as a namespace.
    """
    temp = 37
    kT = (temp + 273.15) * 8.314462175# kT in Joules/mol
    kcal = 4.184E3 # kcal in Joules
    Na = 6.0221412927E23 # Avogadro number
    # cell Volume approximated by a sphere of 7.5 micrometer radius.
    V = 4./3 * pi * 7.5E-6**3
    # concentration to copy number
    c0 = Na*V*1000

    @staticmethod
    def cell_conc(n):
        return n/BioNumbers.V/BioNumbers.Na/1000 # 1000L = 1 m^3

    @staticmethod
    def kcal_to_Kd(E,temp=37):
        kT = (temp + 273.15) * 8.314462175# kT in Joules/mol
        
        # kcal/mol to Kd in nM
        return exp(E*BioNumbers.kcal/kT)*1e9

    @staticmethod
    def Kd_to_kcal(K,temp=37):
        kT = (temp + 273.15) * 8.314462175# kT in Joules/mol
        
        # Kd in nM to E in kcal/mol
        return log(K/1E9)*kT/BioNumbers.kcal
    
class AgoEnergy(object):
    def __init__(self):
        self.E_AGO_mouse = []
        self.E_AGO_fly = []
        self.setup_mir_pairing_energies()

    def setup_mir_pairing_energies(self):
        """ constructed from the supplement of Wee et al. 2012 (Zamore lab) """
        pos = [
            'g1','g1-g2','g2-g3','g3-g4','g4-g5','g5-g6','g6-g7','g7-g8',
            'g8','g9','g8-g9','g9-g10','g10-g11','g12-g13','g13-g14','g15','g16_1','g16_2','g16_3','g14-g15','g15-g16_1','g15-g16_2','g17','g16-g17','g17-g18','g18-g19','g19-g20','g20-g21','g17-g21']

        Kmm = array([17,42,250,320,2400,1100,390,88,74,30,6.0,17,8.3,34,68,12,13,14,7.0,36,20,20,23,22,19,33,19,29,15])
        Kperfect = array([17,26,23,31,32,51,15,24,49,16,15,20,8,20,19,7,4.3,4.3,3.4,10,6.7,5.4,12,6.9,12,22,13,16,11])

        # log-ratios of mismatched and matched Km should correspond to the energy differences
        dE = -log(Kmm/Kperfect)

        Ep = {}
        for p,e,mm in zip(pos,dE,Kmm):
            Ep[p] = e
            #print p,mm,e

        # build a pseudo-energy model per position by averaging the available experimental data
        E = zeros(25)
        E[0] = (Ep['g1'])# + Ep['g1-g2']/2.)/2.
        E[1] = (Ep['g1-g2'] + Ep['g2-g3'])/4.
        E[2] = (Ep['g2-g3'] + Ep['g3-g4'])/4.
        E[3] = (Ep['g3-g4'] + Ep['g4-g5'])/4.
        E[4] = (Ep['g4-g5'] + Ep['g5-g6'])/4.
        E[5] = (Ep['g5-g6'] + Ep['g6-g7'])/4.
        E[6] = (Ep['g6-g7'] + Ep['g7-g8'])/4.
        E[7] = (Ep['g7-g8']/2. + Ep['g8-g9']/2. + Ep['g8'])/3.
        E[8] = (Ep['g8-g9']/2. + Ep['g9-g10']/2. + Ep['g9'])/3.
        E[9] = (Ep['g9-g10'] + Ep['g10-g11'])/4.
        E[10] = Ep['g10-g11'] -E[9]
        E[11] = Ep['g12-g13']/2.
        E[12] = (Ep['g12-g13'] + Ep['g13-g14'])/4.
        E[13] = (Ep['g13-g14'] + Ep['g14-g15'])/4.
        E[14] = (Ep['g14-g15']/2. + Ep['g15-g16_1']/2. + Ep['g15-g16_2']/2. + Ep['g15'])/4.
        E[15] = (Ep['g15-g16_1']/2. +Ep['g15-g16_2']/2. + Ep['g16-g17']/2. + Ep['g16_1'] + Ep['g16_2'] + Ep['g16_3'])/6.
        E[16] = (Ep['g17'] + Ep['g16-g17']/2. + Ep['g17-g18']/2. + Ep['g17-g21']/5.)/4.
        E[17] = (Ep['g17-g18']/2. + Ep['g18-g19']/2. + Ep['g17-g21']/5.)/3.
        E[18] = (Ep['g18-g19']/2. + Ep['g19-g20']/2. + Ep['g17-g21']/5.)/3.
        E[19] = (Ep['g19-g20']/2. + Ep['g20-g21']/2. + Ep['g17-g21']/5.)/3.
        E[20] = (Ep['g20-g21']/2. + Ep['g17-g21']/5.)/2.
        #print E.sum()
        #print E

        # the best fly-Ago Kd is 3.7 pM for a 21nt perfect match.
        # Use that energy to scale the pseudo-energies
        E_fly = BioNumbers.Kd_to_kcal(0.0037,temp=25)
        #print "E_fly",E_fly
        self.E_AGO_fly = E*(E_fly/E.sum())
        
        # now try to estimate a model for mouse (mammals)
        # using the three measured duplexes
        #print "ENERGY MODEL"
        #".))))))).............."
        E_seed = BioNumbers.Kd_to_kcal(0.026,temp=25)
        #".))))))).....))))....."
        E_best = BioNumbers.Kd_to_kcal(0.013,temp=25)
        #"))))))))))))))))))))))"
        E_perfect = BioNumbers.Kd_to_kcal(0.020,temp=25)

        deltaE = E_best - E_seed
        #print "deltaE",deltaE

        # the three guide segments: Seed, supplementary 3' pairing, 
        # and everything else, which in mouse was observed to *reduce* affinity!
        C1 = self.E_AGO_fly[1:8].sum()
        C2 = self.E_AGO_fly[13:17].sum()
        C3 = self.E_AGO_fly.sum()-C1-C2-self.E_AGO_fly[8]

        #print "delta_perf",delta_perf
        s1 = E_seed/C1
        s2 = deltaE/C2
        delta_perf = E_perfect - E_best-self.E_AGO_fly[8]*s1
        s3 = delta_perf/C3

        #print "scaling factors",s1,s2,s3
        # rescale the energy model in each segment to get 
        # an approximate model for mouse Ago
        self.E_AGO_mouse = array(self.E_AGO_fly)
        self.E_AGO_mouse[1:9] = s1*self.E_AGO_fly[1:9]
        self.E_AGO_mouse[13:17] = s2*self.E_AGO_fly[13:17]
        self.E_AGO_mouse[9:13] = s3*self.E_AGO_fly[9:13]
        self.E_AGO_mouse[17:] = s3*self.E_AGO_fly[17:]

    def binding_energy(self,struct,debug=False):
        E = 0.
        if "&" in struct:
            # RNAduplex output. miRNA is second half.
            struct = struct.split("&")[1]

        for e,s in zip(self.E_AGO_mouse,struct):
            if s != ".":
                E += e
                if debug:
                    print s,e
        return E

    def test_run(self):
        print "testing energy model"
        for e,s in [(BioNumbers.Kd_to_kcal(.013,temp=25),"&.))))))).....))))....."),(BioNumbers.Kd_to_kcal(.026,temp=25),"&.))))))).............."),(BioNumbers.Kd_to_kcal(.020,temp=25),"&))))))))))))))))))))))")]:
            print s, "should have ",e
            print self.binding_energy(s)

        E_seed = self.binding_energy(".)))))))..............")
        E_best = self.binding_energy(".))))))).....)))).....")
        E_perfect = self.binding_energy("))))))))))))))))))))))")
        
        E_6mer = self.binding_energy(".((((((")
        E_7mer = self.binding_energy(".(((((((")
        E_8mer = self.binding_energy(".((((((((",debug=True)
        print "energies",E_6mer,E_7mer,E_8mer
        
        for i,e in zip(arange(1,26),self.E_AGO_mouse):
            print i,e

        K_6mer = BioNumbers.kcal_to_Kd(E_6mer,temp=37)
        K_7mer = BioNumbers.kcal_to_Kd(E_7mer,temp=37)
        K_8mer = BioNumbers.kcal_to_Kd(E_8mer,temp=37)

        print "Kd_6-mer @37C %.4f nM" % K_6mer
        print "Kd_7-mer @37C %.4f nM" % K_7mer
        print "Kd_8-mer @37C %.4f nM" % K_8mer
        
        print "testing original Zamore lab measurements"
        print "Kd_seed @25C %.4f nM" % BioNumbers.kcal_to_Kd(E_seed,temp=25)
        print "Kd_best @25C %.4f nM" % BioNumbers.kcal_to_Kd(E_best,temp=25)
        print "Kd_perfect @25C %.4f nM" % BioNumbers.kcal_to_Kd(E_perfect,temp=25)
        
        from pylab import plot,axhline,xlabel,ylabel,legend,show
        plot(arange(1,26),self.E_AGO_fly,drawstyle='steps-mid',label="estimated E_i of fly Ago")
        plot(arange(1,26),self.E_AGO_mouse,color="k",drawstyle='steps-mid',label="estimated E_i of mouse Ago")
        axhline(0,color="k",linestyle="--")
        ylabel("E_i [kcal/mol]")
        xlabel("guide RNA position [nt]")
        legend()
        show()

AGO = AgoEnergy()
#AGO.test_run()

def solve_self_consistency(Kd,m,N,epsilon=1e-6,M=None):
    """
    Uses the monotonic feature of the self-consistency equations (1,2) to solve 
    them by recursively splitting the interval of Theta (total fraction of bound
    sites), until the relative error is within tolerance.
    """
    if not M:
        M = m.sum()
        
    tmax = m.sum()/M
    #print tmax
    n_est = 1

    def evaluate(theta):
        f = max(0,N - theta*M)
        #print f
        return (theta*M - (m*f/(f+Kd)).sum())

    def iteration(theta_min,theta_max,err1,err3,i):
        center = (theta_max + theta_min)/2

        if i < n_est:
            dy = err3-err1
            dx = theta_max - theta_min

            guess = theta_min - dx/dy * err1
            mid = ((n_est-i)*guess+center)/(n_est-i+1)
            #mid = guess
        else:
            mid = center

        dTheta = fabs(theta_min - theta_max)
        #print "iteration",i,dTheta/mid
        if dTheta/mid < epsilon:
            #print "needed %d iterations" % i
            return mid

        err2 = evaluate(mid)
        #print theta_min,theta_max,err1,err3,mid,err2

        if err1 < 0 and err2 > 0:
            return iteration(theta_min,mid,err1,err2,i+1)
        elif err2 < 0 and err3 > 0:
            return iteration(mid,theta_max,err2,err3,i+1)
        else:
            print "non-monotonous function!"
            return mid

    start,end = 0.,tmax
    return iteration(start,end,evaluate(start),evaluate(end),0)


class CellModel(object):
    """
    Model to describe a pool of binding sites with 
    diss. constants {k_i} and concentrations {c_i}
    """
    
    def __init__(self,k_i=[],c_i=[],total_conc=0,**kwargs):
        self.k_i = k_i
        self.c_i = c_i
        self.e_i = CellModel.Kd_to_kcal(k_i)

        if not total_conc:
            total_conc = c_i.sum()
        else:
            self.c_i *= float(total_conc)/c_i.sum()
        self.total_conc = total_conc

    def fraction_bound_at_conc(self,conc):
        """
        self-consistently compute the fraction of binding agent that is bound
        at the given concentration, in equilibrium. Takes either a fixed 
        concentration, or a range (any iterable) and returns either a number or
        vector, accordingly.
        """
        if hasattr(conc,"__iter__"):
            return array([solve_self_consistency(self.k_i,self.c_i,c) for c in conc])
        else:
            return solve_self_consistency(self.k_i,self.c_i,conc)
        
    def free_at_conc(self,conc):
        theta = self.fraction_bound_at_conc(conc)
        return conc - theta*self.total_conc

    def occupancies_at_conc(self,conc):
        """
        Uses the self-consistent computation of *free* binding agent, at the
        given total concentration, to compute the equilibrium occupancies for 
        all k_i
        """
        f = self.free_at_conc(conc)

        # local binding equation
        if hasattr(conc,"__iter__"):
            F = f[:,newaxis]
            return F/(F+self.k_i[newaxis,:])
        else:
            return f/(f+self.k_i)

    @staticmethod
    def copies_to_concentrations(n,volume=BioNumbers.V):
        """
        Takes copy-numbers and converts to nano-molars based on 
        the cell (cytoplasmic) volume.
        (default=sphere of 7.5 micrometer radius).
        """
        return n/volume/BioNumbers.Na/1000/1E-9 # 1000L = 1 m^3, 1nM = 1E-9 M

    @staticmethod
    def concentrations_to_copies(c,volume=BioNumbers.V):
        """
        Takes nano-molars and converts to copy-numbers based on 
        the cell (cytoplasmic) volume.
        (default=sphere of 7.5 micrometer radius).
        """
        return c*volume*BioNumbers.Na*1E-6 # 1000L = 1 m^3, 1nM = 1E-9 M

    @staticmethod
    def kcal_to_Kd(E,temp=37):
        """
        Takes a vector of binding energies in kcal/mol and converts to
        dissociation constants (in nM) at the given (Celsius!) temperature 
        (default=37 degrees C).
        """
        kT = (temp + 273.15) * 8.314462175# kT in Joules/mol
        # kcal/mol to Kd in nM
        return exp(E*BioNumbers.kcal/kT)*1e9

    @staticmethod
    def Kd_to_kcal(K,temp=37):
        """
        Inverse of kcal_to_Kd
        """
        kT = (temp + 273.15) * 8.314462175# kT in Joules/mol
        
        # Kd in nM to E in kcal/mol
        return log(K/1E9)*kT/BioNumbers.kcal



def mirna_concentrations(fname,n_risc=150000):
    """
    Takes read counts for miRNA sequences (e.g. from a sequencing run)
    and assigns concentrations, based on the assumption that:
    
      * read count is proportional to abundance
      * all miRNAs are loaded
      * the total amount of AGO/RISC is constant and known 
        (150000/cell by default)
      * there is something like a "normal" cell (BioNumbers)
      * expression profiles are comparable across cells
      
    Each and every of these points may be open to debate but it should give
    a reasonable order of magnitude.
    """
      
    total = 0.
    expr = {}
    for line in file(fname):
        mir,reads = line.split('\t')
        expr[mir] = float(reads)
        total += expr[mir]
        
    for k in expr.keys():
        expr[k] = CellModel.copies_to_concentrations(expr[k]*n_risc/total)
        
    return expr
    
def model_from_file(fname,**kwargs):
    e_i = []
    m_i = []
    c_i = []
    for line in file(fname):
        if line.startswith('#'):
            continue
        e,m,c = line.split('\t')
        e_i.append(float(e))
        m_i.append(float(m))
        c_i.append(float(c))

    k_i = CellModel.kcal_to_Kd(array(e_i))

    return CellModel(k_i,array(c_i),**kwargs)


E_6mer = AGO.binding_energy("&.((((((")
E_7mer = AGO.binding_energy("&.(((((((")
E_8mer = AGO.binding_energy("&.((((((((")

K_6mer = CellModel.kcal_to_Kd(E_6mer,temp=options.T)
K_7mer = CellModel.kcal_to_Kd(E_7mer,temp=options.T)
K_8mer = CellModel.kcal_to_Kd(E_8mer,temp=options.T)

print "Kd_6-mer @%.1fC %.4f nM" % (options.T,K_6mer)
print "Kd_7-mer @%.1fC %.4f nM" % (options.T,K_7mer)
print "Kd_8-mer @%.1fC %.4f nM" % (options.T,K_8mer)


# simplified model with only 6-mers, 7-mers and 8-mers
if options.simple:
    # these are simply the seed matches in each transcript, expression weighted, 
    # per million transcripts
    mer8 = 112915.99
    mer7 = 196187.84 - mer8 # 7-mers are included in 8mers
    mer6 = 603937.55 - mer7 - mer8 # 6-mers are included in 7mers and 8mers

    # we add in a "test" binding site at index 0 with the best (predicted) Kd, 
    # but zero abundance. this is expected in some of the ploting later, but does 
    # not affect the equilibrium at all.
    c_i = CellModel.copies_to_concentrations(array([0,mer8,mer7,mer6]) / 4) #250,000 mRNAs
    k_i = array([mir20a.k_i.min(),K_8mer,K_7mer,K_6mer])

    mir20a = CellModel(k_i,c_i)
    print "approximate TA [nM] in simplified model %.2f" % (mir20a.total_conc)
    # end of simplified model
else:
    mir20a = model_from_file(options.energies)

    # the default model was computed assuming 250,000 mRNAs
    mir20a.c_i *= float(options.M/250000.) 
    # let's assume only 50% accessibility
    mir20a.c_i *= .5
    mir20a.total_conc = mir20a.c_i.sum()

    print "strongest binder in the set",mir20a.k_i.min()
    
TA = CellModel.concentrations_to_copies(mir20a.c_i.take((mir20a.k_i < 10.).nonzero()).sum())
print "approximate TA for miR-20a in monocytes",TA
TA_str = "B_%.1f" % TA

figure()
title("Kd distribution")
semilogx(mir20a.k_i,mir20a.c_i,linestyle="steps")
savefig("mir20a_Kd_%s.%s" % (TA_str,options.format))



# concentration range [nM], exponential not linear
conc = 10**arange(-2,4,.1)

figure()
title("free versus total miRNA")
free = mir20a.free_at_conc(conc)
loglog(conc,free,"k",label="full model")
loglog(conc,conc,"gray",linestyle="dashed",label="no competition")
xlabel("total miR-20a concentration [nM]")
ylabel("free miR-20a concentration [nM]")
ylim(1E-6,1E4)
savefig("free_vs_total_%s.%s" % (TA_str,options.format))




# estimate miR-20a abundance from sequencing reads.
mirs = mirna_concentrations(options.mirnas,n_risc=options.RISC)

m20a = mirs['UAAAGUGCUUAUAGUGCAGGUAGU'] + \
       mirs['UAAAGUGCUUAUAGUGCAGGUA'] + \
       mirs['UAAAGUGCUUAUAGUGCAGGU'] + \
       mirs['UAAAGUGCUUAUAGUGCAGG']

print "m20a",m20a


# investigate the impact of competition on the occupancies of test-binding sites.
figure()
# concentration range [nM], exponential not linear
c = 10**arange(-4,4,.1)
occ = transpose(mir20a.occupancies_at_conc(c))
f = mir20a.free_at_conc(c)

ka = K_8mer
kb = K_7mer
kc = K_6mer

print "test k's for occ_by_total_examples.pdf",[ka,kb,kc]
semilogx(c,f/(f+ka),"k")
semilogx(c,f/(f+kb),color="#666666")
semilogx(c,f/(f+kc),color="#a7a7a7")
xlabel("total miR-20a concentration [nM]")

semilogx(c,c/(c+ka),linestyle="dashed",color="black")
semilogx(c,c/(c+kb),linestyle="dashed",color="#666666")
semilogx(c,c/(c+kc),linestyle="dashed",color="#a7a7a7")
xticks([0.01,0.1,1,10,100],["0.01","0.1","1","10","100"])
yticks([0,.25,.5,.75,1],["0%","25%","50%","75%","100%"])
subplots_adjust(hspace=0.1,bottom=0.5)

k0 = K_8mer #mir20a.k_i.min()
P75 = 3*k0 + (mir20a.c_i * (3*k0/(mir20a.k_i + 3*k0))).sum()

axvline(P75,color='r', linestyle='dashed')
axvline(m20a,color='k',linestyle='dotted')
savefig("occ_by_total_examples_%s.%s" % (TA_str,options.format))


# test what happens when a "sponge" RNA with many binding sites is expressed
def sponge(model,cs):
    """
    takes a model and adds cs to the concentration of the best binding site 
    (sorted, so k_i[0] = min(k_i))
    returns new model with altered site concentrations.
    """
    c_i = array(model.c_i)
    c_i[0] += cs
    return CellModel(model.k_i,c_i)

figure()
title("free miRNA changes upon sponge expression")
loglog(conc,mir20a.free_at_conc(conc),"k")
# concentration of miR-20a at which the strongest site is at 75% occupancy
# i.e. free miR-20a = 3*min(Ki)
print "best Kd=%f. 75%% occupancy at F=%f instead of P=%f" % (k0,P75,3*k0)

s1 = sponge(mir20a,CellModel.copies_to_concentrations(500))
s10 = sponge(mir20a,CellModel.copies_to_concentrations(5000))
s100 = sponge(mir20a,CellModel.copies_to_concentrations(20000))
loglog(conc,s1.free_at_conc(conc),":",color="#d4aa00")
loglog(conc,s10.free_at_conc(conc),"--",color="#ff6600")
loglog(conc,s100.free_at_conc(conc),"-",color="#d40000")

c1 = P75
c2 = 2*P75
c3 = 10*P75
axvline(c3,color="blue")
axvline(c2,color="red")
axvline(c1,color="black")

xlim(1,1E3)
ylim(1E-4,1E3)
savefig("sponge_shifts_%s.%s" % (TA_str,options.format))

# test how this change in free miRNA concentration translates into altered site occupancies
figure()
#P75 *=2
f = mir20a.free_at_conc(P75)
f_s1 = s1.free_at_conc(P75)
f_s10 = s10.free_at_conc(P75)
f_s100 = s100.free_at_conc(P75)

bar(arange(3),[f/(f+K_8mer),f/(f+K_7mer),f/(f+K_6mer)],width=.2,color="k",label="no sponge")
bar(arange(3)+.21,[f_s1/(f_s1+K_8mer),f_s1/(f_s1+K_7mer),f_s1/(f_s1+K_6mer)],width=.2,color="#d4aa00",label="+500 sites")
bar(arange(3)+.42,[f_s10/(f_s10+K_8mer),f_s10/(f_s10+K_7mer),f_s10/(f_s10+K_6mer)],width=.2,color="#ff6600",label="+5,000 sites")
bar(arange(3)+.63,[f_s100/(f_s100+K_8mer),f_s100/(f_s100+K_7mer),f_s100/(f_s100+K_6mer)],width=.2,color="#d40000",label="+20,000 sites")

ylabel("binding site occupancy")
legend(loc='upper right')
savefig("occ_shifts_by_sponge_%s.%s" % (TA_str,options.format))




def deltas(wt_model,sponges,conc=50,color="k"):

    wt = wt_model.occupancies_at_conc(conc)
    s1,s10,s100 = sponges
    d1 = wt - s1.occupancies_at_conc(conc)
    d10 = wt - s10.occupancies_at_conc(conc)
    d100 = wt - s100.occupancies_at_conc(conc)
    
    k_i = wt_model.k_i
    semilogx(k_i,d1,":"+color)
    semilogx(k_i,d10,"--"+color)
    semilogx(k_i,d100,"-"+color)

# plot the absolute reductions in site occupancies due to sponge expression (not shown in paper)
figure()    
deltas(mir20a,[s1,s10,s100],conc=c3,color="b")
deltas(mir20a,[s1,s10,s100],conc=c2,color="r")
deltas(mir20a,[s1,s10,s100],conc=c1,color="k")
xlim(.01,1000)
savefig("deltas_%s.%s" % (TA_str,options.format))


# look at different initial occupancies (not shown in paper)
def T_for_occ(occ,k0,model=mir20a):
    f = occ/(1-occ)
    return k0 * f + (mir20a.c_i * (f*k0/(mir20a.k_i + f*k0))).sum()

figure()
# unperturbed free protein concentrations at which an 8-mer is at x percent occupancy
T50 = T_for_occ(.5,K_8mer)
T90 = T_for_occ(.9,K_8mer)
T10 = T_for_occ(.1,K_8mer)

print T10,T50,T90
TA50 = T50 - mir20a.free_at_conc(T50)
TA10 = T10 - mir20a.free_at_conc(T10)
TA90 = T90 - mir20a.free_at_conc(T90)
print TA10,TA50,TA90


dF50 = []
dF10 = []
dF90 = []

dO50 = []
dO10 = []
dO90 = []

cs = 10**arange(-2,4,.1)
for c in cs:
    s = sponge(mir20a,c)
    f50 = s.free_at_conc(T50)
    f90 = s.free_at_conc(T90)
    f10 = s.free_at_conc(T10)
    dF50.append(K_8mer - f50)
    dF90.append(K_8mer*(.9/.1) - f90)
    dF10.append(K_8mer*(.1/.9) - f10)
    
    dO50.append(f50/(K_8mer + f50))
    dO10.append(f10/(K_8mer + f10))
    dO90.append(f90/(K_8mer + f90))
    
semilogx(cs,dO10,":",color="#d4aa00",label="8mer at 10%")
semilogx(cs,dO50,"--",color="#ff6600",label="8mer at 50%")
semilogx(cs,dO90,"-",color="#d40000",label="8mer at 90%")
axhline(.1)
axhline(.5)
axhline(.9)

savefig("deltaF_vs_sponge_expression_%s.%s" % (TA_str,options.format))
